﻿-- PugLax = CreateFrame("Frame","PugLax",UIParent)
PugLax = LibStub("AceAddon-3.0"):NewAddon("PugLax", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local PugLax = _G.PugLax
local GroupTalents = LibStub("LibGroupTalents-1.0")

-- [[ Start Addon Variables ]] --

PugLax.loc = "enUS"
if (PugLocs[GetLocale()]) then
	PugLax.loc = GetLocale()
else
	PugLax.loc = "enUS"
end

PugLax.supportsSRTI = true

PugLax.ccData = { 
		polymorph={ spellID=118, level=14, class="MAGE", favmark="moon"}, 
		freezingTrap={ spellID=1499, level=28, class="HUNTER", favmark="square"}, 
		hex={ spellID=51514, level=80, class="SHAMAN", favmark="diamond"}, 
		bindElemental={ spellID=76780, level=64, class="SHAMAN"}, 
		sap={ spellID=6770, level=10, class="ROGUE", favmark="circle"}, 
		shackleUndead={ spellID=9484, level=32, class="PRIEST"}, 
		mindControl={ spellID=605, level=38, class="PRIEST"},
		entanglingRoots={ spellID=339, level=7, class="DRUID"}, 
		hibernate={ spellID=2637, level=48, class="DRUID"}, 
		cyclone={ spellID=33786, level=74, class="DRUID"}, 
		fear={ spellID=5782, level=14, class="WARLOCK"}, 
		seduction={ spellID=6358, level=20, class="WARLOCK"}, 
		banish={ spellID=710, level=32, class="WARLOCK", favmark="diamond"}, 
		wyvernSting={ spellID=19386, class="HUNTER"}, 
		repentance={ spellID=20066, class="PALADIN"},
	}

PugLax.GroupData = { }

PugLax.AvailableCC = { }

-- To adjust the mark priority, reorder these (so that your highest value mark is towards the top of this array)

PugLax.Marks = {
		skull={ markindex=8, markstring="rt8", assignment=nil },
		cross={ markindex=7,markstring="rt7", assignment=nil },
		star={ markindex=1, markstring="rt1", assignment=nil},
		moon={ markindex=5, markstring="rt5", assignment=nil},
		square={ markindex=6, markstring="rt6", assignment=nil},
		circle={ markindex=2, markstring="rt2", assignment=nil},
		diamond={ markindex=3, markstring="rt3", assignment=nil},
		triangle={ markindex=4, markstring="rt4", assignment=nil},
	}
	
PugLax.SortedMarks = {
	"skull",
	"cross",
	"star",
	"moon",
	"square",
	"circle",
	"diamond",
	"triangle",
}

-- [[ End Addon Variables ]] --

-- [[ Start Addon Event Handlers ]] --

function PugLax:PARTY_MEMBERS_CHANGED()
	local inTeam = (GetNumPartyMembers() > 0)
	if inTeam then
		if not PugLax.inTeam then
			-- PugLax:Reset(PugLocs[PugLax.loc]["userReset"])
			PugLax:FinishReset()
			
			if PugLax.config.jumpMenu.autoCollapse == true then
				PugLax:ToggleGUI("expand")
			elseif PugLax.config.jumpMenu.autoHide == true then
				PugLax:ToggleGUI("show")
			end
		end
	else
		if PugLax.inTeam then
			if PugLax.config.jumpMenu.autoCollapse == true or PugLax.config.jumpMenu.autoCollapse == nil then
				PugLax:ToggleGUI("collapse")
			elseif PugLax.config.jumpMenu.autoHide == true or PugLax.config.jumpMenu.autoHide == nil then
				PugLax:ToggleGUI("hide")
			end
		end
	end
	
	PugLax.inTeam = inTeam
	
	PugLax.GroupData = { }
	
	if (GetNumRaidMembers() == 0) then
		PugLax:AddParty();
	else
		PugLax:AddRaid();
	end
	
	PugLax:BuildCC()
end

function PugLax:ACTIVE_TALENT_GROUP_CHANGED()
	if (self.talentChangeTimer) then
		self:CancelTimer(self.talentChangeTimer, true)
	end
	
	self:ScheduleTimer("FinishTalentChange", 2, {})
end

function PugLax:FinishTalentChange()
	PugLax:CleanAssignments()
	PugLax:AddCC("player")
	PugLax:RebuildGUI()
end

function PugLax:ZONE_CHANGED_NEW_AREA()
	PugLax.ccPriority = PugLax:GetCurrentPriority()
end

function PugLax:AddParty()
	for i=1,4,1 do
		if (GetPartyMember(i)) then
			local localClass, englishClass = UnitClass("party"..i)
			-- PugLax.GroupData[i] = { name = UnitName("party"..i), class=englishClass, level=UnitLevel("party"..i), unitid="party"..i }
			PugLax.GroupData[UnitName("party"..i)] = { class=englishClass, level=UnitLevel("party"..i), unitid="party"..i }
		end
	end
end

function PugLax:AddRaid()
	for i=1,40,1 do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		if (name) then
			local localClass, englishClass = UnitClass("raid"..i)
			-- PugLax.GroupData[i] = { name = UnitName("raid"..i), class=englishClass, level=UnitLevel("raid"..i),  unitid="raid"..i }
			PugLax.GroupData[UnitName("raid"..i)] = { class=englishClass, level=UnitLevel("raid"..i),  unitid="raid"..i }
		end
	end
end

function PugLax:OnInitialize()
	if not PugLax.ccPriority then
		PugLax.ccPriority = PugLax:GetCurrentPriority()
	end
	
	PugLax.db = LibStub("AceDB-3.0"):New("PugLaxDB", PugLax.configDefaults, true)
	PugLax.db.RegisterCallback(self, "OnDatabaseShutdown", "RefreshDatabase")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("PugLax", PugLax.Options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("PugLax", "PugLax")
	
	PugLax.config = { }
	PugLax.config.marks = {}
	PugLax.config.cc = {}
	PugLax.config.jumpMenu = {}
	PugLax.config.jumpMenu.header = {}
	PugLax.config.jumpMenu.header.color = {}
	PugLax.config.jumpMenu.header.borderColor = {}
	PugLax.config.jumpMenu.header.insets = {}
	PugLax.config.jumpMenu.rows = {}
	PugLax.config.jumpMenu.rows.color = {}
	PugLax.config.jumpMenu.rows.borderColor = {}
	PugLax.config.jumpMenu.rows.insets = {}
	PugLax.config.killorder = {}
	
	PugLax.config.marks.skull = PugLax.db.global.marks.skull
	PugLax.config.marks.cross = PugLax.db.global.marks.cross
	PugLax.config.marks.star = PugLax.db.global.marks.star
	PugLax.config.marks.moon = PugLax.db.global.marks.moon
	PugLax.config.marks.circle = PugLax.db.global.marks.circle
	PugLax.config.marks.square = PugLax.db.global.marks.square
	PugLax.config.marks.diamond = PugLax.db.global.marks.diamond
	PugLax.config.marks.triangle = PugLax.db.global.marks.triangle
	
	PugLax.config.cc.polymorph = PugLax.db.global.cc.polymorph
	PugLax.config.cc.freezingTrap = PugLax.db.global.cc.freezingTrap
	PugLax.config.cc.sap = PugLax.db.global.cc.sap
	PugLax.config.cc.hex = PugLax.db.global.cc.hex
	PugLax.config.cc.repentance = PugLax.db.global.cc.repentance
	PugLax.config.cc.entanglingRoots = PugLax.db.global.cc.entanglingRoots
	PugLax.config.cc.wyvernSting = PugLax.db.global.cc.wyvernSting
	PugLax.config.cc.banish = PugLax.db.global.cc.banish
	PugLax.config.cc.bindElemental = PugLax.db.global.cc.bindElemental
	PugLax.config.cc.shackleUndead = PugLax.db.global.cc.shackleUndead
	PugLax.config.cc.hibernate = PugLax.db.global.cc.hibernate
	PugLax.config.cc.fear = PugLax.db.global.cc.fear
	PugLax.config.cc.seduction = PugLax.db.global.cc.seduction
	PugLax.config.cc.cyclone = PugLax.db.global.cc.cyclone
	PugLax.config.cc.mindControl = PugLax.db.global.cc.mindControl
	
	PugLax.config.jumpMenu.tukui = PugLax.db.global.jumpMenu.tukui
	PugLax.config.jumpMenu.collapse = PugLax.db.global.jumpMenu.collapse
	PugLax.config.jumpMenu.hide = PugLax.db.global.jumpMenu.hide
	PugLax.config.jumpMenu.lock = PugLax.db.global.jumpMenu.lock
	PugLax.config.jumpMenu.growUp = PugLax.db.global.jumpMenu.growUp
	PugLax.config.jumpMenu.autoHide = PugLax.db.global.jumpMenu.autoHide
	PugLax.config.jumpMenu.autoCollapse = PugLax.db.global.jumpMenu.autoCollapse
	
	PugLax.config.jumpMenu.header.color = PugLax.db.global.jumpMenu.header.color
	PugLax.config.jumpMenu.header.height = PugLax.db.global.jumpMenu.header.height
	PugLax.config.jumpMenu.header.width = PugLax.db.global.jumpMenu.header.width
	PugLax.config.jumpMenu.header.font = PugLax.db.global.jumpMenu.header.font
	PugLax.config.jumpMenu.header.fontColor = PugLax.db.global.jumpMenu.header.fontColor
	PugLax.config.jumpMenu.header.fontSize = PugLax.db.global.jumpMenu.header.fontSize
	PugLax.config.jumpMenu.header.texture = PugLax.db.global.jumpMenu.header.texture
	PugLax.config.jumpMenu.header.border = PugLax.db.global.jumpMenu.header.border
	PugLax.config.jumpMenu.header.borderColor = PugLax.db.global.jumpMenu.header.borderColor
	PugLax.config.jumpMenu.header.edgeSize = PugLax.db.global.jumpMenu.header.edgeSize
	PugLax.config.jumpMenu.header.insets = PugLax.db.global.jumpMenu.header.insets
	PugLax.config.jumpMenu.header.iconScale = PugLax.db.global.jumpMenu.header.iconScale
	
	PugLax.config.jumpMenu.rows.spacing = PugLax.db.global.jumpMenu.rows.spacing
	PugLax.config.jumpMenu.rows.color = PugLax.db.global.jumpMenu.rows.color
	PugLax.config.jumpMenu.rows.height = PugLax.db.global.jumpMenu.rows.height
	PugLax.config.jumpMenu.rows.font = PugLax.db.global.jumpMenu.rows.font
	PugLax.config.jumpMenu.rows.fontSize = PugLax.db.global.jumpMenu.rows.fontSize
	PugLax.config.jumpMenu.rows.texture = PugLax.db.global.jumpMenu.rows.texture
	PugLax.config.jumpMenu.rows.border = PugLax.db.global.jumpMenu.rows.border
	PugLax.config.jumpMenu.rows.borderColor = PugLax.db.global.jumpMenu.rows.borderColor
	PugLax.config.jumpMenu.rows.edgeSize = PugLax.db.global.jumpMenu.rows.edgeSize
	PugLax.config.jumpMenu.rows.insets = PugLax.db.global.jumpMenu.rows.insets
	PugLax.config.jumpMenu.rows.iconScale = PugLax.db.global.jumpMenu.rows.iconScale
	
	PugLax.config.killorder[1] = PugLax.db.global.killorder[1] 
	PugLax.config.killorder[2]  = PugLax.db.global.killorder[2]
	PugLax.config.killorder[3]  = PugLax.db.global.killorder[3] 
	
	PugLax.config.silentInstanceReset = PugLax.db.global.silentInstanceReset
	
	PugLax:BuildCC()
	PugLax:RebuildGUI()
	
	if PugLax.config.jumpMenu.autoCollapse == true or PugLax.config.jumpMenu.autoCollapse == nil then
		PugLax:ToggleGUI("collapse")
	elseif PugLax.config.jumpMenu.autoHide == true or PugLax.config.jumpMenu.autoHide == nil then
		PugLax:ToggleGUI("hide")
	end
end

function PugLax:RefreshDatabase()
	PugLax.db.global.marks.skull = PugLax.config.marks.skull
	PugLax.db.global.marks.cross = PugLax.config.marks.cross
	PugLax.db.global.marks.star = PugLax.config.marks.star
	PugLax.db.global.marks.moon = PugLax.config.marks.moon
	PugLax.db.global.marks.circle = PugLax.config.marks.circle
	PugLax.db.global.marks.square = PugLax.config.marks.square
	PugLax.db.global.marks.diamond = PugLax.config.marks.diamond
	PugLax.db.global.marks.triangle = PugLax.config.marks.triangle
	
	PugLax.db.global.cc.polymorph = PugLax.config.cc.polymorph
	PugLax.db.global.cc.freezingTrap = PugLax.config.cc.freezingTrap
	PugLax.db.global.cc.sap = PugLax.config.cc.sap
	PugLax.db.global.cc.hex = PugLax.config.cc.hex
	PugLax.db.global.cc.repentance = PugLax.config.cc.repentance
	PugLax.db.global.cc.entanglingRoots = PugLax.config.cc.entanglingRoots
	PugLax.db.global.cc.wyvernSting = PugLax.config.cc.wyvernSting
	PugLax.db.global.cc.banish = PugLax.config.cc.banish
	PugLax.db.global.cc.bindElemental = PugLax.config.cc.bindElemental
	PugLax.db.global.cc.shackleUndead = PugLax.config.cc.shackleUndead
	PugLax.db.global.cc.hibernate = PugLax.config.cc.hibernate
	PugLax.db.global.cc.fear = PugLax.config.cc.fear
	PugLax.db.global.cc.seduction = PugLax.config.cc.seduction
	PugLax.db.global.cc.cyclone = PugLax.config.cc.cyclone
	PugLax.db.global.cc.mindControl = PugLax.config.cc.mindControl
	
	PugLax.db.global.jumpMenu.tukui = PugLax.config.jumpMenu.tukui
	PugLax.db.global.jumpMenu.collapse = PugLax.config.jumpMenu.collapse
	PugLax.db.global.jumpMenu.hide = PugLax.config.jumpMenu.hide
	PugLax.db.global.jumpMenu.lock = PugLax.config.jumpMenu.lock
	PugLax.db.global.jumpMenu.growUp = PugLax.config.jumpMenu.growUp
	PugLax.db.global.jumpMenu.autoHide = PugLax.config.jumpMenu.autoHide
	PugLax.db.global.jumpMenu.autoCollapse = PugLax.config.jumpMenu.autoCollapse
	
	PugLax.db.global.jumpMenu.header.color = PugLax.config.jumpMenu.header.color
	PugLax.db.global.jumpMenu.header.height = PugLax.config.jumpMenu.header.height
	PugLax.db.global.jumpMenu.header.width = PugLax.config.jumpMenu.header.width
	PugLax.db.global.jumpMenu.header.font = PugLax.config.jumpMenu.header.font
	PugLax.db.global.jumpMenu.header.fontColor = PugLax.config.jumpMenu.header.fontColor
	PugLax.db.global.jumpMenu.header.fontSize = PugLax.config.jumpMenu.header.fontSize
	PugLax.db.global.jumpMenu.header.texture = PugLax.config.jumpMenu.header.texture
	PugLax.db.global.jumpMenu.header.border = PugLax.config.jumpMenu.header.border
	PugLax.db.global.jumpMenu.header.borderColor = PugLax.config.jumpMenu.header.borderColor
	PugLax.db.global.jumpMenu.header.edgeSize = PugLax.config.jumpMenu.header.edgeSize
	PugLax.db.global.jumpMenu.header.insets = PugLax.config.jumpMenu.header.insets
	PugLax.db.global.jumpMenu.header.iconScale = PugLax.config.jumpMenu.header.iconScale
	
	PugLax.db.global.jumpMenu.rows.spacing = PugLax.config.jumpMenu.rows.spacing
	PugLax.db.global.jumpMenu.rows.color = PugLax.config.jumpMenu.rows.color
	PugLax.db.global.jumpMenu.rows.height = PugLax.config.jumpMenu.rows.height
	PugLax.db.global.jumpMenu.rows.font = PugLax.config.jumpMenu.rows.font
	PugLax.db.global.jumpMenu.rows.fontSize = PugLax.config.jumpMenu.rows.fontSize
	PugLax.db.global.jumpMenu.rows.texture = PugLax.config.jumpMenu.rows.texture
	PugLax.db.global.jumpMenu.rows.border = PugLax.config.jumpMenu.rows.border
	PugLax.db.global.jumpMenu.rows.borderColor = PugLax.config.jumpMenu.rows.borderColor
	PugLax.db.global.jumpMenu.rows.edgeSize = PugLax.config.jumpMenu.rows.edgeSize
	PugLax.db.global.jumpMenu.rows.insets = PugLax.config.jumpMenu.rows.insets
	PugLax.db.global.jumpMenu.rows.iconScale = PugLax.config.jumpMenu.rows.iconScale
	
	PugLax.db.global.killorder[1] = PugLax.config.killorder[1]
	PugLax.db.global.killorder[2] = PugLax.config.killorder[2]
	PugLax.db.global.killorder[3] = PugLax.config.killorder[3]
	
	PugLax.db.global.silentInstanceReset = PugLax.config.silentInstanceReset
	
end

function PugLax:OnEnable()

end

function PugLax:OnDisable()

end

-- [[ End Addon Event Handlers ]] --

-- [[ Start Addon Slash Handlers ]] --

function PugLax:PugSlash(input)
	if(input == "party") then
		PugLax:EchoAll()
	elseif(input == "zone") then
		local zonetext = GetRealZoneText()
		print(zonetext)
	elseif(input == "priority") then
		if PugLax.UsedPriority then
			print("Used Priority: " .. PugLax.UsedPriority)
		end
		PugLax:PrintPriority()
	elseif(input == "babble") then
		local B = LibStub("LibBabble-Zone-3.0")
		local BR = B:GetReverseLookupTable()
		if (BR[GetRealZoneText()]) then
			print(BR[GetRealZoneText()] .. " is localized!")
		else
			print("Couldn't find current zone, using default priority")
		end
	elseif(input == "rebuild") then
		PugLax:Reset(PugLocs[PugLax.loc]["userReset"])
	elseif(input == "center") then
		PugLax_Header:ClearAllPoints()
		PugLax_Header:SetPoint("CENTER")
	elseif(input == "lock") then
		if PugLax.config.jumpMenu.lock then
			PugLax.config.jumpMenu.lock = false
			PugLax_Header:RegisterForDrag("LeftButton")
		else
			PugLax.config.jumpMenu.lock = true
			PugLax_Header:RegisterForDrag(nil)
		end
	else
		PugLax:ToggleGUI()
	end
end

-- [[ End Addon Slash Handlers ]] --

-- [[ Start Addon Functions ]] --

function KillOrderToString()
	local killString = ""
	local didStart = false
	
	for i = 1, 3, 1 do
		if PugLax.config.killorder[i] then
			if didStart then
				killString = killString .. "->"
			else
				didStart = true
			end
			
			killString = killString .. "{" .. PugLax.Marks[PugLax.config.killorder[i]]["markstring"] .. "}"
		end
	end
		
	return killString
end

function PugLax:EchoAll()
	local channel
	if (GetNumRaidMembers() > 0) then
		channel = "RAID"
	else
		channel = "PARTY"
	end
	
	-- for k,v in pairs(PugLax.Marks) do
		-- if v["assignment"] then
			-- PugLax:EchoRow(k, channel)
		-- end
	-- end
	
	for i,v in ipairs(PugLax.SortedMarks) do
		if PugLax.Marks[v]["assignment"] then
			PugLax:EchoRow(v, channel)
		end
	end
	
	SendChatMessage(PugLocs[PugLax.loc]["echoKillOrder"] .. KillOrderToString() , channel, nil, nil)
end

function PugLax:EchoRow(rowname)
	local channel
	if (GetNumRaidMembers() > 0) then
		channel = "RAID"
	else
		channel = "PARTY"
	end
	
	if (PugLax.Marks[rowname]["assignment"]) then
		--local msg = PugLax.Marks[rowname]["assignment"]["playername"] .. PugLocs[PugLax.loc]["echoPlease"] .. PugLocs[PugLax.loc]["ccShortName"][PugLax.Marks[rowname]["assignment"]["ccKey"]] .. PugLocs[PugLax.loc]["echoThe"] .. "{" .. PugLax.Marks[rowname]["markstring"] .. "}"
		
		local msg = PugLax.Marks[rowname]["assignment"]["playername"] .. PugLocs[PugLax.loc]["echoPlease"] .. PugLax:SpellLink(PugLax.ccData[PugLax.Marks[rowname]["assignment"]["ccKey"]]["spellID"]) .. PugLocs[PugLax.loc]["echoThe"] .. "{" .. PugLax.Marks[rowname]["markstring"] .. "}"
		
		SendChatMessage(msg, channel, nil, nil)
	end
end

function PugLax:SpellLink(spellID)
	local spellName = GetSpellInfo(spellID)
	return "\124cff71d5ff\124Hspell:" .. spellID .. "\124h[" .. spellName .. "]\124h\124r";
	--"\124cff71d5ff\124Hspell:51514\124h[Hex]\124h\124r"
end

function PugLax:EchoTimer()
	local channel
	if (GetNumRaidMembers() == 0) then
		channel = "PARTY"
	else
		channel = "RAID"
	end
	
	if (self.pullCountdownTimer) then
		self:CancelTimer(self.pullCountdownTimer, true)
	end
	self.pullCountdownVal = 3
	self.pullCountdownChannel = channel
	SendChatMessage(PugLocs[PugLax.loc]["pullannounce"]["pullingin"] .. self.pullCountdownVal .. "...", channel, nil, nil)
	self.pullCountdownTimer = self:ScheduleRepeatingTimer("HandleTimer", 1)
end

function PugLax:HandleTimer()
	self.pullCountdownVal = self.pullCountdownVal - 1
	if (self.pullCountdownVal <= 0) then
		SendChatMessage(PugLocs[PugLax.loc]["pullannounce"]["pullingnow"], self.pullCountdownChannel, nil, nil)
		self:CancelTimer(self.pullCountdownTimer)
	else
		SendChatMessage(self.pullCountdownVal .. "...", self.pullCountdownChannel, nil, nil)
	end
end

function PugLax:Reset(message)
	if PugLax.ResetFrame then
		PugLax.ResetFrame.FontString:SetText(message)
		PugLax.ResetFrame:Show()
	else
		PugLax:FinishReset()
	end
end

function PugLax:FinishReset()
	PugLax.BuildingInProgress = true
	PugLax.ccPriority = nil
	PugLax.ccPriority = PugLax:GetCurrentPriority()
	PugLax:ClearMarks()
	PugLax:BuildCC()
	PugLax:AssignCC()
	PugLax:RebuildGUI()
	PugLax.BuildingInProgress = false
end

function PugLax:BuildCC()
	PugLax.ccPriority = PugLax:GetCurrentPriority()
	PugLax:CleanAssignments()
	
	PugLax:AddCC("player")
	
	for k,v in pairs(PugLax.GroupData) do
		PugLax:AddCC(v.unitid)
	end
	
	PugLax:RebuildGUI()
end

function PugLax:CleanAssignments()
	indToClean = { }
	for i,v in pairs(PugLax.AvailableCC) do
		local ownerName = v["playername"]
		if  PugLax.GroupData[ownerName] or ownerName == UnitName("player") then
			--print("Keeping cc "..v["ccKey"])
			
			if not PugLax.ccData[v["ccKey"]]["level"] then
				if v["mark"] then
					v["mark"]["assignment"] = nil
				end
				table.insert(indToClean, i)
				PugLax:CheckTalent({unitname=v["playername"], ccKey=v["ccKey"], retry=0})
			end
			
		else
			--print("Cleaning cc "..v["ccKey"] .. ownerName)
			if v["mark"] then
				v["mark"]["assignment"] = nil
			end
			table.insert(indToClean, i)
		end
	end
	
	for i,v in pairs(indToClean) do
		table.remove(PugLax.AvailableCC, v)
	end
end

function PugLax:AddCC(unitid)
	if not unitid then
		print("PUGLAX ERROR: AddCC called without a valid unitid!")
		return
	end
	
	if unitid ~= "player" and GetUnitName(unitid) == GetUnitName("player") then
		return
	end
	
	if UnitLevel(unitid) < 1 then
		self["addCCTimer"..unitid] = self:ScheduleTimer("DelayAddCC", 1, {["unitid"]=unitid, ["unitname"]=UnitName(unitid)})
		return
	end
	
	for i,v in ipairs(PugLax.ccPriority) do
		--if (PugLax.config.cc[v] == true) then
			local locClass, englishClass = UnitClass(unitid)
			if (PugLax.ccData[v]["level"]) then
				if (UnitLevel(unitid) >= PugLax.ccData[v]["level"] and englishClass == PugLax.ccData[v]["class"]) then
					--table.insert(PugLax.AvailableCC, { ccKey=v, playername=UnitName(unitid), mark=nil })
					PugLax:CreateCC(v, UnitName(unitid), false)
				end
			elseif (PugLax.ccData[v]["spellID"]) then
				local talentName = GetSpellInfo(PugLax.ccData[v]["spellID"])
				if UnitLevel(unitid) >= 10 and englishClass == PugLax.ccData[v]["class"] then
					PugLax:CreateCC(v, UnitName(unitid), true)
					PugLax:CheckTalent({unitname=GetUnitName(unitid), ccKey=v, retry=0})
				end
			end
		--end
	end
end

function PugLax:CheckTalent(info)
	local unitid
	
	if info.unitname == UnitName("player") then
		unitid = "player"
	elseif PugLax.GroupData[info.unitname] then
		unitid = PugLax.GroupData[info.unitname]["unitid"]
	end
	
	if unitid then
		local talentname = GetSpellInfo(PugLax.ccData[info.ccKey]["spellID"])
		if (not UnitIsDead(unitid)) and GroupTalents:UnitHasTalent(unitid, talentname) then
			PugLax:CreateCC(info.ccKey, info.unitname, false)
		else
			if UnitIsDead(unitid) or not CheckInteractDistance(unitid, 1) then
				self["CheckTalentTimer"..info.unitname..info.ccKey] = self:ScheduleTimer("CheckTalent", 3, info)
			elseif info.retry < 5 then
				info.retry = info.retry + 1
				self["CheckTalentTimer"..info.unitname..info.ccKey] = self:ScheduleTimer("CheckTalent", 1, info)
			end
		end
	end
end

function PugLax:DelayAddCC(unitinfo)
	if UnitName(unitinfo.unitid) == unitinfo.unitname then
		PugLax:AddCC(unitinfo.unitid)
		PugLax:RebuildGUI()
	end
end

function PugLax:CreateCC(newKey, newPlayer, newLocked)
	local exists = false
	local shouldAssign = false
	
	for i,v in ipairs(PugLax.AvailableCC) do
		if v["playername"] == newPlayer and v["ccKey"] == newKey then
			
			shouldAssign = v["locked"] and not newLocked
			
			v["locked"] = newLocked
			exists = true
			break
		end
	end
	
	if not exists then
		table.insert(PugLax.AvailableCC, { ccKey=newKey, playername=newPlayer, mark=nil, locked=newLocked})		
		shouldAssign = not PugLax.BuildingInProgress
	end
	
	if shouldAssign then
		PugLax:AssignCC()
	end
	
	PugLax:RebuildGUI()
end

function PugLax:AssignCC()
	-- Handle special cases first
	for i,v in ipairs(PugLax.AvailableCC) do
		if not v["locked"] then
			if PugLax.config.cc[v["ccKey"]] then
				if(v["mark"] == nil and v["ccKey"] == "polymorph" and PugLax.Marks["moon"]["assignment"] == nil  and PugLax.config.marks["moon"]) then
					v["mark"] = PugLax.Marks["moon"]
					PugLax.Marks["moon"]["assignment"] = v
				end
				
				if(v["mark"] == nil and v["ccKey"] == "sap" and PugLax.Marks["circle"]["assignment"] == nil  and PugLax.config.marks["circle"]) then
					v["mark"] = PugLax.Marks["circle"]
					PugLax.Marks["circle"]["assignment"] = v
				end
				
				if(v["mark"] == nil and v["ccKey"] == "freezingTrap" and PugLax.Marks["square"]["assignment"] == nil  and PugLax.config.marks["square"]) then
					v["mark"] = PugLax.Marks["square"]
					PugLax.Marks["square"]["assignment"] = v
				end
			end
		end
	end
	
	-- Handle remaining in order of priority
	for ia,va in ipairs(PugLax.ccPriority) do
		for ib,vb in ipairs(PugLax.AvailableCC) do
			if (vb["mark"] == nil and va == vb["ccKey"] and PugLax.config.cc[vb["ccKey"]] and not vb["locked"]) then
				vb["mark"] = PugLax:GetNextAvailableMark(vb)
			end
		end
	end
end

function PugLax:GetNextAvailableMark(assignment)
	-- if (PugLax.Marks["moon"]["assignment"] == nil and PugLax.config.marks["moon"]) then
		-- PugLax.Marks["moon"]["assignment"] = assignment
		-- return PugLax.Marks["moon"]
	-- elseif (PugLax.Marks["square"]["assignment"] == nil and PugLax.config.marks["square"]) then
		-- PugLax.Marks["square"]["assignment"] = assignment
		-- return PugLax.Marks["square"]
	-- elseif (PugLax.Marks["circle"]["assignment"] == nil and PugLax.config.marks["circle"]) then
		-- PugLax.Marks["circle"]["assignment"] = assignment
		-- return PugLax.Marks["circle"]
	-- elseif (PugLax.Marks["diamond"]["assignment"] == nil and PugLax.config.marks["diamond"]) then
		-- PugLax.Marks["diamond"]["assignment"] = assignment
		-- return PugLax.Marks["diamond"]
	-- elseif (PugLax.Marks["triangle"]["assignment"] == nil and PugLax.config.marks["triangle"]) then
		-- PugLax.Marks["triangle"]["assignment"] = assignment
		-- return PugLax.Marks["triangle"]
	-- elseif (PugLax.Marks["star"]["assignment"] == nil and PugLax.config.marks["star"]) then
		-- PugLax.Marks["star"]["assignment"] = assignment
		-- return PugLax.Marks["star"]
	-- end
	
	-- for k,v in pairs(PugLax.Marks) do
		-- if v["assignment"] == nil and PugLax.config.marks[k] then
			-- PugLax.Marks[k]["assignment"] = assignment
			-- return PugLax.Marks[k]
		-- end
	-- end
	
	for i,v in ipairs(PugLax.SortedMarks) do
		if PugLax.Marks[v]["assignment"] == nil and PugLax.config.marks[v] then
			PugLax.Marks[v]["assignment"] = assignment
			return PugLax.Marks[v]
		end
	end
end

function PugLax:ClearMarks()
	for k,v in pairs(PugLax.Marks) do
		if v["assignment"] then
			v["assignment"]["mark"] = nil
		end
		
		v["assignment"] = nil
	end
end

function PugLax:GetCurrentPriority()	
	local B = LibStub("LibBabble-Zone-3.0")
	local BR = B:GetReverseLookupTable()
	if (BR[GetRealZoneText()] and PugLaxPriorityLists[BR[GetRealZoneText()]]) then
		
		if PugLax.inTeam and PugLax.ccPriority ~= nil and PugLax.ccPriority ~= PugLaxPriorityLists[BR[GetRealZoneText()]] then
			if PugLax.config.silentInstanceReset then
				PugLax:FinishReset()
			else
				PugLax:Reset(PugLocs[PugLax.loc]["zoneReset"])
			end
		end
		PugLax.UsedPriority = BR[GetRealZoneText()]
		return PugLaxPriorityLists[BR[GetRealZoneText()]]
	else
		if PugLax.ccPriority then
			PugLax.UsedPriority = PugLax.UsedPriority
			return PugLax.ccPriority
		else
			PugLax.UsedPriority = "DEFAULT PRIORITY"
			return PugLaxDefaultPriority
		end
	end
end

function PugLax:PrintPriority()
	for i,v in ipairs(PugLax.ccPriority) do
		print(i .. " / " .. GetSpellInfo(PugLax.ccData[v]["spellID"]))
	end
end

-- [[ End Addon Functions ]] --

-- [[ Start Addon Options ]] --


-- [[ End Addon Options ]] --

-- [[ Start Addon JumpMenu ]] --



-- [[ End Addon JumpMenu ]] --

-- [[ Start Ace3 Registers ]] --

PugLax:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
PugLax:RegisterEvent("PARTY_MEMBERS_CHANGED")
PugLax:RegisterEvent("ZONE_CHANGED_NEW_AREA")
PugLax:RegisterChatCommand("puglax", "PugSlash")
PugLax:RegisterChatCommand("plax", "PugSlash")
PugLax:RegisterChatCommand("pl", "PugSlash")

-- [[ End Ace3 Registers ]] --