local PugLax = _G.PugLax

local LSM = LibStub("LibSharedMedia-3.0")

local marksTable = {
	["skull"] = PugLocs[PugLax.loc]["markStrings"]["skull"],
	["cross"] = PugLocs[PugLax.loc]["markStrings"]["cross"],
	["star"] = PugLocs[PugLax.loc]["markStrings"]["star"],
	["triangle"] = PugLocs[PugLax.loc]["markStrings"]["triangle"],
	["diamond"] = PugLocs[PugLax.loc]["markStrings"]["diamond"],
	["circle"] = PugLocs[PugLax.loc]["markStrings"]["circle"],
	["square"] = PugLocs[PugLax.loc]["markStrings"]["square"],
	["moon"] = PugLocs[PugLax.loc]["markStrings"]["moon"],
}

PugLax.configDefaults = {
	global = {
		saveVersion = 0,
		silentInstanceReset = false,
		killorder = {
			"skull",
			"cross",
			"triangle",
		},
		marks = {
			skull = false,
			cross = false,
			star = false,
			moon = true,
			circle = true,
			square = true,
			diamond = true,
			triangle = true,
		},
		cc = {
			polymorph = true,
			freezingTrap = true,
			sap = true,
			hex = true,
			repentance = true,
			entanglingRoots = true,
			wyvernSting = true,
			banish = true,
			bindElemental = true,
			shackleUndead = true,
			hibernate = true,
			fear = true,
			seduction = true,
			cyclone = true,
			mindControl = true,
		},
		jumpMenu = {
			lock = false,
			tukui = true,
			collapse = false,
			hide = false,
			growUp = false,
			autoHide = false,
			autoCollapse = true,
			header = {
				iconScale = 0.6,
				["color"] = {
					["a"] = 0.9000000059604645,
					["b"] = 0.1686274509803922,
					["g"] = 0.09803921568627451,
					["r"] = 0.07843137254901961,
				},
				height = 24,
				width = 172,
				font = "Friz Quadrata TT",
				fontColor= {r=1,g=1,b=1,a=1},
				fontSize = 11,
				texture="Blizzard",
				border="Blizzard Tooltip",
				["borderColor"] = {
					["a"] = 1,
					["b"] = 0.5607843137254902,
					["g"] = 0.3254901960784314,
					["r"] = 0.3882352941176471,
				},
				["edgeSize"] = 16,
				["insets"] = {
					["top"] = 4,
					["right"] = 4,
					["left"] = 4,
					["bottom"] = 4,
				},
			},
			rows = {
				iconScale = 0.6,
				spacing=-3,
				["color"] = {
					["a"] = 0.9000000059604645,
					["b"] = 0.1686274509803922,
					["g"] = 0.09803921568627451,
					["r"] = 0.07843137254901961,
				},
				height = 24,
				font = "Friz Quadrata TT",
				fontSize = 11,
				texture="Blizzard",
				border="Blizzard Tooltip",
				["borderColor"] = {
					["a"] = 1,
					["b"] = 0.5372549019607843,
					["g"] = 0.3686274509803922,
					["r"] = 0.3607843137254902,
				},
				["edgeSize"] = 16,
				["insets"] = {
					["top"] = 4,
					["right"] = 4,
					["left"] = 4,
					["bottom"] = 4,
				},
			}
		}
	}
}

PugLax.Options = {
	name=PugLocs[PugLax.loc]["options"]["name"],
	handler= PugLax,
	type='group',
	args = {
		general = {
			name = PugLocs[PugLax.loc]["options"]["general"],
			type = 'group',
			order = 1,
			args = {
				growUp = {
					name = PugLocs[PugLax.loc]["options"]["growUp"],
					type = 'toggle',
					order = 1,
					set = function(info, val) PugLax.config.jumpMenu.growUp = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.growUp end
				},
				tukui = {
					name = PugLocs[PugLax.loc]["options"]["tukui"],
					type = 'toggle',
					order = 2,
					set = function(info, val) PugLax.config.jumpMenu.tukui = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.tukui end
				},
				silentInstanceReset = {
					name = PugLocs[PugLax.loc]["options"]["silentInstanceReset"],
					desc = PugLocs[PugLax.loc]["options"]["silentInstanceResetDesc"],
					type = 'toggle',
					order = 3,
					set = function(info, val) PugLax.config.silentInstanceReset = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.silentInstanceReset end
				},
				autoHeader = {
					name = PugLocs[PugLax.loc]["options"]["autoHeader"],
					type = 'header',
					order = 4,
				},
				autoDesc = {
					name = PugLocs[PugLax.loc]["options"]["autoDesc"],
					type = 'description',
					order = 5,
				},
				autoHide = {
					name = PugLocs[PugLax.loc]["options"]["autoHide"],
					type = 'toggle',
					order = 6,
					tristate = true,
					set = function(info, val)
							if val ~= false then
								PugLax.config.jumpMenu.autoCollapse = false;
							end
							PugLax.config.jumpMenu.autoHide = val;
						end,
					get = function(info) return PugLax.config.jumpMenu.autoHide end,
				},
				autoCollapse = {
					name = PugLocs[PugLax.loc]["options"]["autoCollapse"],
					type = 'toggle',
					order = 7,
					tristate = true,
					set = function(info, val)
							if val ~= false then
								PugLax.config.jumpMenu.autoHide = false;
							end
							PugLax.config.jumpMenu.autoCollapse = val;
						end,
					get = function(info) return PugLax.config.jumpMenu.autoCollapse end,
				}
			}
		},
		killOrder = {
			name = PugLocs[PugLax.loc]["options"]["killOrder"],
			type = 'group',
			order = 2,
			args = {
				first = {
					type = 'select',
					name = PugLocs[PugLax.loc]["options"]["killOrderFirst"],
					values = marksTable,
					set = function(info, val) PugLax.config.killorder[1] = val; end,
					get = function(info) return PugLax.config.killorder[1] end,
				},
				second = {
					type = 'select',
					name = PugLocs[PugLax.loc]["options"]["killOrderSecond"],
					values = marksTable,
					set = function(info, val) PugLax.config.killorder[2] = val; end,
					get = function(info) return PugLax.config.killorder[2] end,
				},
				third = {
					type = 'select',
					name = PugLocs[PugLax.loc]["options"]["killOrderThird"],
					values = marksTable,
					set = function(info, val) PugLax.config.killorder[3] = val; end,
					get = function(info) return PugLax.config.killorder[3] end,
				},
			},
		},
		enableMarks = {
			name = PugLocs[PugLax.loc]["options"]["enabledMarks"],
			type = 'group',
			order = 3,
			args = {
				skull = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["skull"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["skull"],
					set = function(info, val) PugLax.config.marks["skull"] = val; PugLax:BuildCC(); PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.marks["skull"] end
				},
				cross = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["cross"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["cross"],
					set = function(info, val) PugLax.config.marks["cross"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["cross"] end
				},
				star = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["star"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["star"],
					set = function(info, val) PugLax.config.marks["star"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["star"] end
				},
				moon = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["moon"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["moon"],
					set = function(info, val) PugLax.config.marks["moon"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["moon"] end
				},
				circle = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["circle"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["circle"],
					set = function(info, val) PugLax.config.marks["circle"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["circle"] end
				},
				square = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["square"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["square"],
					set = function(info, val) PugLax.config.marks["square"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["square"] end
				},
				diamond = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["diamond"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["diamond"],
					set = function(info, val) PugLax.config.marks["diamond"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["diamond"] end
				},
				triangle = {
					type = 'toggle',
					name = PugLocs[PugLax.loc]["markStrings"]["triangle"],
					desc = PugLocs[PugLax.loc]["options"]["echoEnableUsing"] .. PugLocs[PugLax.loc]["markStrings"]["triangle"],
					set = function(info, val) PugLax.config.marks["triangle"] = val; PugLax:BuildCC(); PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.marks["triangle"] end
				}
			}
		},
		enableCC = {
			name = PugLocs[PugLax.loc]["options"]["enabledCC"],
			type = 'group',
			order = 4,
			args = {
				polymorph = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["polymorph"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["polymorph"]["spellID"]),
					set = function(info, val) PugLax.config.cc["polymorph"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["polymorph"] end
				},
				freezingTrap = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["freezingTrap"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["freezingTrap"]["spellID"]),
					set = function(info, val) PugLax.config.cc["freezingTrap"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["freezingTrap"] end
				},
				sap = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["sap"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["sap"]["spellID"]),
					set = function(info, val) PugLax.config.cc["sap"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["sap"] end
				},
				hex = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["hex"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["hex"]["spellID"]),
					set = function(info, val) PugLax.config.cc["hex"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["hex"] end
				},
				repentance = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["repentance"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["repentance"]["spellID"]),
					set = function(info, val) PugLax.config.cc["repentance"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["repentance"] end
				},
				entanglingRoots = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["entanglingRoots"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["entanglingRoots"]["spellID"]),
					set = function(info, val) PugLax.config.cc["entanglingRoots"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["entanglingRoots"] end
				},
				wyvernSting = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["wyvernSting"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["wyvernSting"]["spellID"]),
					set = function(info, val) PugLax.config.cc["wyvernSting"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["wyvernSting"] end
				},
				banish = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["banish"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["banish"]["spellID"]),
					set = function(info, val) PugLax.config.cc["banish"] = val end,
					get = function(info) return PugLax.config.cc["banish"] end
				},
				bindElemental = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["bindElemental"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["bindElemental"]["spellID"]),
					set = function(info, val) PugLax.config.cc["bindElemental"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["bindElemental"] end
				},
				shackleUndead = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["shackleUndead"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["shackleUndead"]["spellID"]),
					set = function(info, val) PugLax.config.cc["shackleUndead"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["shackleUndead"] end
				},
				hibernate = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["hibernate"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["hibernate"]["spellID"]),
					set = function(info, val) PugLax.config.cc["hibernate"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["hibernate"] end
				},
				fear = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["fear"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["fear"]["spellID"]),
					set = function(info, val) PugLax.config.cc["fear"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["fear"] end
				},
				seduction = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["seduction"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["seduction"]["spellID"]),
					set = function(info, val) PugLax.config.cc["seduction"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["seduction"] end
				},
				Cyclone = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["cyclone"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["cyclone"]["spellID"]),
					set = function(info, val) PugLax.config.cc["cyclone"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["cyclone"] end
				},
				mindControl = {
					type = 'toggle',
					name = GetSpellInfo(PugLax.ccData["mindControl"]["spellID"]),
					desc = PugLocs[PugLax.loc]["options"]["echoEnableCC"] .. GetSpellInfo(PugLax.ccData["mindControl"]["spellID"]),
					set = function(info, val) PugLax.config.cc["mindControl"] = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.cc["mindControl"] end
				}
			}
		},
		header = {
			name = PugLocs[PugLax.loc]["options"]["jumpMenuHeader"],
			type = 'group',
			order = 5,
			args = {
				sizeHeader = {
					name = PugLocs[PugLax.loc]["options"]["size"],
					type = 'header',
					order = 0,
				},
				iconScale = {
					name = PugLocs[PugLax.loc]["options"]["iconScale"],
					type = 'range',
					order = 1,
					min = 0.1,
					max = 2.0,
					step = 0.1,
					set = function(info, val) PugLax.config.jumpMenu.header.iconScale = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.iconScale end
				},
				height = {
					name = PugLocs[PugLax.loc]["options"]["height"],
					type = 'range',
					order = 2,
					min = 10,
					max = 50,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.height = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.height end
				},
				width = {
					name = PugLocs[PugLax.loc]["options"]["width"],
					type = 'range',
					order = 3,
					min = 50,
					max = 500,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.width = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.width end
				},
				fontHeader = {
					name = PugLocs[PugLax.loc]["options"]["font"],
					type = 'header',
					order = 4,
				},
				font = {
					type = 'select',
					order = 5,
					dialogControl = 'LSM30_Font',
					name = PugLocs[PugLax.loc]["options"]["font"],
					values = LSM:HashTable("font"),
					set = function(info, val) PugLax.config.jumpMenu.header.font = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.font end
				},
				fontSize = {
					name = PugLocs[PugLax.loc]["options"]["size"],
					type = 'range',
					order = 6,
					min = 4,
					max = 32,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.fontSize = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.fontSize end
				},
				bgHeader = {
					name = PugLocs[PugLax.loc]["options"]["background"],
					type = 'header',
					order = 7,
				},
				texture = {
					type = 'select',
					dialogControl = 'LSM30_Statusbar',
					order = 8,
					name = PugLocs[PugLax.loc]["options"]["texture"],
					values = LSM:HashTable("statusbar"),
					set = function(info, val) PugLax.config.jumpMenu.header.texture = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.texture end
				},
				color = {
					type = 'color',
					name = PugLocs[PugLax.loc]["options"]["color"],
					hasAlpha = true,
					order = 9,
					set = function(info, r, g, b, a)
							PugLax.config.jumpMenu.header.color.r = r
							PugLax.config.jumpMenu.header.color.g = g
							PugLax.config.jumpMenu.header.color.b = b
							PugLax.config.jumpMenu.header.color.a = a
							PugLax:RebuildGUI()
						end,
					get = function(info) return unpack(colorArray(PugLax.config.jumpMenu.header.color)) end
				},
				borderHeader = {
					name = PugLocs[PugLax.loc]["options"]["border"],
					type = 'header',
					order = 10,
				},
				borderTexture = {
					type = 'select',
					dialogControl = 'LSM30_Border',
					order = 11,
					name = PugLocs[PugLax.loc]["options"]["border"],
					values = LSM:HashTable("border"),
					set = function(info, val) PugLax.config.jumpMenu.header.border = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.border end
				},
				borderColor = {
					type = 'color',
					name = PugLocs[PugLax.loc]["options"]["color"],
					hasAlpha = true,
					order = 12,
					set = function(info, r, g, b, a)
							PugLax.config.jumpMenu.header.borderColor.r = r
							PugLax.config.jumpMenu.header.borderColor.g = g
							PugLax.config.jumpMenu.header.borderColor.b = b
							PugLax.config.jumpMenu.header.borderColor.a = a
							PugLax:RebuildGUI()
						end,
					get = function(info) return unpack(colorArray(PugLax.config.jumpMenu.header.borderColor)) end
				},
				edgeSize = {
					name = PugLocs[PugLax.loc]["options"]["edgesize"],
					type = 'range',
					order = 13,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.edgeSize = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.edgeSize end
				},
				insetHeader = {
					name = PugLocs[PugLax.loc]["options"]["border"].." "..PugLocs[PugLax.loc]["options"]["insets"],
					type = 'header',
					order = 14,
				},
				edgeInsetLeft = {
					name = PugLocs[PugLax.loc]["options"]["left"],
					type = 'range',
					order = 15,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.insets.left = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.insets.left end
				},
				edgeInsetRight = {
					name = PugLocs[PugLax.loc]["options"]["right"],
					type = 'range',
					order = 16,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.insets.right = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.insets.right end
				},
				edgeInsetTop = {
					name = PugLocs[PugLax.loc]["options"]["top"],
					type = 'range',
					order = 17,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.insets.top = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.insets.top end
				},
				edgeInsetBottom = {
					name = PugLocs[PugLax.loc]["options"]["bottom"],
					type = 'range',
					order = 18,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.header.insets.bottom = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.header.insets.bottom end
				},
			}
		},
		rows = {
			name = PugLocs[PugLax.loc]["options"]["jumpMenuRows"],
			type = 'group',
			order = 6,
			args = {
				sizeHeader = {
					name = PugLocs[PugLax.loc]["options"]["size"],
					type = 'header',
					order = 0,
				},
				iconScale = {
					name = PugLocs[PugLax.loc]["options"]["iconScale"],
					type = 'range',
					order = 1,
					min = 0.1,
					max = 2.0,
					step = 0.1,
					set = function(info, val) PugLax.config.jumpMenu.rows.iconScale = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.iconScale end
				},
				spacing = {
					name = PugLocs[PugLax.loc]["options"]["spacing"],
					type = 'range',
					order = 2,
					min = -50,
					max = 50,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.spacing = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.spacing end
				},
				height = {
					name = PugLocs[PugLax.loc]["options"]["height"],
					type = 'range',
					order = 3,
					min = 10,
					max = 50,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.height = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.height end
				},
				fontHeader = {
					name = PugLocs[PugLax.loc]["options"]["font"],
					type = 'header',
					order = 4,
				},
				font = {
					type = 'select',
					order = 5,
					dialogControl = 'LSM30_Font',
					name = PugLocs[PugLax.loc]["options"]["font"],
					values = LSM:HashTable("font"),
					set = function(info, val) PugLax.config.jumpMenu.rows.font = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.font end
				},
				fontSize = {
					name = PugLocs[PugLax.loc]["options"]["size"],
					type = 'range',
					order = 6,
					min = 4,
					max = 32,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.fontSize = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.fontSize end
				},
				bgHeader = {
					name = PugLocs[PugLax.loc]["options"]["background"],
					type = 'header',
					order = 7,
				},
				texture = {
					type = 'select',
					dialogControl = 'LSM30_Statusbar',
					order = 8,
					name = PugLocs[PugLax.loc]["options"]["texture"],
					values = LSM:HashTable("statusbar"),
					set = function(info, val) PugLax.config.jumpMenu.rows.texture = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.texture end
				},
				color = {
					type = 'color',
					name = PugLocs[PugLax.loc]["options"]["color"],
					hasAlpha = true,
					order = 9,
					set = function(info, r, g, b, a)
							PugLax.config.jumpMenu.rows.color.r = r
							PugLax.config.jumpMenu.rows.color.g = g
							PugLax.config.jumpMenu.rows.color.b = b
							PugLax.config.jumpMenu.rows.color.a = a
							PugLax:RebuildGUI()
						end,
					get = function(info) return unpack(colorArray(PugLax.config.jumpMenu.rows.color)) end
				},
				brdHeader = {
					name = PugLocs[PugLax.loc]["options"]["border"],
					type = 'header',
					order = 10,
				},
				borderTexture = {
					type = 'select',
					dialogControl = 'LSM30_Border',
					order = 11,
					name = PugLocs[PugLax.loc]["options"]["border"],
					values = LSM:HashTable("border"),
					set = function(info, val) PugLax.config.jumpMenu.rows.border = val; PugLax:RebuildGUI()  end,
					get = function(info) return PugLax.config.jumpMenu.rows.border end
				},
				borderColor = {
					type = 'color',
					name = PugLocs[PugLax.loc]["options"]["color"],
					hasAlpha = true,
					order = 12,
					set = function(info, r, g, b, a)
							PugLax.config.jumpMenu.rows.borderColor.r = r
							PugLax.config.jumpMenu.rows.borderColor.g = g
							PugLax.config.jumpMenu.rows.borderColor.b = b
							PugLax.config.jumpMenu.rows.borderColor.a = a
							PugLax:RebuildGUI()
						end,
					get = function(info) return unpack(colorArray(PugLax.config.jumpMenu.rows.borderColor)) end
				},
				edgeSize = {
					name = PugLocs[PugLax.loc]["options"]["edgesize"],
					type = 'range',
					order = 13,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.edgeSize = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.edgeSize end
				},
				sizeHeader = {
					name = PugLocs[PugLax.loc]["options"]["border"].." "..PugLocs[PugLax.loc]["options"]["insets"],
					type = 'header',
					order = 14,
				},
				edgeInsetLeft = {
					name = PugLocs[PugLax.loc]["options"]["left"],
					type = 'range',
					order = 15,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.insets.left = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.insets.left end
				},
				edgeInsetRight = {
					name = PugLocs[PugLax.loc]["options"]["right"],
					type = 'range',
					order = 16,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.insets.right = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.insets.right end
				},
				edgeInsetTop = {
					name = PugLocs[PugLax.loc]["options"]["top"],
					type = 'range',
					order = 17,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.insets.top = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.insets.top end
				},
				edgeInsetBottom = {
					name = PugLocs[PugLax.loc]["options"]["bottom"],
					type = 'range',
					order = 18,
					min = -20,
					max = 20,
					step = 1,
					set = function(info, val) PugLax.config.jumpMenu.rows.insets.bottom = val; PugLax:RebuildGUI() end,
					get = function(info) return PugLax.config.jumpMenu.rows.insets.bottom end
				},
			}
		}
	}
}

function colorArray(color)
	local a = { color.r, color.g, color.b, color.a }
	return a
end